#include "connectioncache.hh"

const Connection &ConnectionCache::GetConnection(unsigned ConnID)
{
    if(ConnID >= recs.size())return recs[0].Conn; /* FIXME: This may cause crashes. */
    return recs[ConnID].Conn;
}

unsigned ConnectionCache::GetConnectionID(const Connection &conn, SockProtocol proto)
{
    static const unsigned EXPIRELEN = 3600;
    time_t Now = time(NULL);
    int port = conn.DestPort;
    
    map<Connection, unsigned>::iterator i = key2rec.find(conn);
    if(i != key2rec.end())
    {
        recs[i->second].LastTouch = Now;
        return i->second;
    }
    
    unsigned ConnID;
    /* Find out if there's an expiring connection */
    for(ConnID=0; ConnID<recs.size(); ++ConnID)
    {
        if(!recs[ConnID].Expiring)continue;
        if(Now > recs[ConnID].LastTouch + EXPIRELEN)
        {
            if(recs[ConnID].LastTouch > 0)
                DeleteConnectionID(ConnID);
            break;
        }
    }
    if(ConnID >= recs.size())
    {
        recs.reserve((ConnID + 63) & ~63);
        recs.resize(ConnID + 1);
    }
    
    recs[ConnID].LastTouch = Now;
    recs[ConnID].Expiring  = (proto != SockProtocolTCP);
    recs[ConnID].Conn      = conn;
    
    owner->AddOpeningMsg(ConnID, proto, port);
    
    key2rec.insert(make_pair(conn, ConnID));
    return ConnID;
}

void ConnectionCache::DeleteConnectionID(unsigned ConnID)
{
    if(ConnID >= recs.size())return;
    
    owner->AddClosingMsg(ConnID);
    
    map<Connection, unsigned>::iterator i = key2rec.find(recs[ConnID].Conn);
    if(i != key2rec.end())
        key2rec.erase(i);
    
    recs[ConnID].LastTouch = 0;
    recs[ConnID].Expiring  = true;
}

ConnectionCache::ConnectionCache(struct StdinType *o) : owner(o)
{
}

ConnectionCache::~ConnectionCache()
{
}
