#include "mastermsg.hh"

DatatableType Int2Data(unsigned n)
{
    Datatype Arr[4] = 
    {
        n & 0xFF,
        (n >> 8) & 0xFF,
        (n >> 16) & 0xFF,
        (n >> 24) & 0xFF
    };
    return DatatableType(Arr, sizeof Arr);
}

unsigned Data2Int(const DatatableType &b, unsigned pos)
{
    return (unsigned)b[pos+0]
        | ((unsigned)b[pos+1] << 8)
        | ((unsigned)b[pos+2] << 16)
        | ((unsigned)b[pos+3] << 24);
}

MasterMsg::MasterMsg(enum MasterMsg::MsgType Typ) : Type(Typ)
{
}

MasterMsg::MasterMsg(const DatatableType &s)
{
    UnmakeString(s);
}


void MasterMsg::SetParam(enum MasterMsg::ParamType Typ, const DatatableType &data)
{
    params[Typ] = data;
}

#include <stdio.h>
void MasterMsg::Debug() const
{
    fprintf(stderr, "   Type=");
    switch(Type)
    {
        case MsgTypeListen: fprintf(stderr, "Listen"); break;
        case MsgTypeOpen:   fprintf(stderr, "Open"); break;
        case MsgTypeData:   fprintf(stderr, "Data"); break;
        case MsgTypeClose:  fprintf(stderr, "Close"); break;
    }
    fprintf(stderr, " (%u)\n", (unsigned)Type);
    
    for(paramap::const_iterator i=params.begin(); i!=params.end(); ++i)
    {
        fprintf(stderr, "   Param: %u:", i->first);
        switch(i->first)
        {
            case ParamTypeData:   fprintf(stderr, "Data(%s)", i->second.c_str()); break;
            case ParamTypeConnID: fprintf(stderr, "ConnID(%u)", Data2Int(i->second)); break;
            case ParamTypeProto:  fprintf(stderr, "Proto(%s)", i->second.c_str()); break;
            case ParamTypePort:   fprintf(stderr, "Proto(%u)", Data2Int(i->second)); break;
        }
        fprintf(stderr, "\n");
    }
    fprintf(stderr, "\n");
}

const DatatableType MasterMsg::MakeString() const
{
    DatatableType t;
    
    t += Int2Data((unsigned)Type);
    
    paramap::const_iterator i;
    for(i=params.begin(); i!=params.end(); ++i)
    {
        t += Int2Data(i->first);
        t += Int2Data(i->second.size());
        t += i->second;
    }
    
    return t;
}

void MasterMsg::UnmakeString(const DatatableType &s)
{
    params.clear();
    if(s.size() < 4)return;
    
    Type = (MsgType)Data2Int(s, 0);
    
    for(unsigned a=4; a+8 <= s.size(); )
    {
        unsigned key = Data2Int(s, a); a += 4;
        unsigned len = Data2Int(s, a); a += 4;
        if(a+len > s.size())break;
        
        SetParam((ParamType)key, s.substr(a, len));
        a += len;
    }

#if 1
    fprintf(stderr, "Received a message:\n");
    Debug();
#endif
}

char MasterMsg::GetCharParam(ParamType key) const
{
    paramap::const_iterator i = params.find(key);
    if(i == params.end())return '\0';
    if(i->second.size() < 1)return '\0';
    return i->second[0];
}

unsigned MasterMsg::GetIntParam(ParamType key) const
{
    paramap::const_iterator i = params.find(key);
    if(i == params.end())return 0;
    if(i->second.size() < 4)return 0;
    return Data2Int(i->second);
}

DatatableType MasterMsg::GetDataParam(ParamType key) const
{
    paramap::const_iterator i = params.find(key);
    if(i == params.end())return DatatableType();
    return i->second;
}
