#include <map>
#include <string>

using std::map;
using std::basic_string;

#include "buffer.hh"

typedef basic_string<Datatype> DatatableType;


extern unsigned Data2Int(const DatatableType &b, unsigned pos=0);
extern DatatableType Int2Data(unsigned n);


class MasterMsg
{
public:
    enum MsgType
    {
        MsgTypeListen=0,
        MsgTypeOpen=1,
        MsgTypeData=2,
        MsgTypeClose=3
    };
    enum ParamType
    {
        ParamTypeData=0,   /* Array of octets                           */
        ParamTypeConnID=1, /* little-endian (386 style) 32-bit unsigned */
        ParamTypeProto=2,  /* char: 'T'=TCP, 'U'=udp                    */
        ParamTypePort=3    /* little-endian (386 style) 32-bit unsigned */
    };

public:
    MasterMsg(enum MsgType Typ);
    MasterMsg(const DatatableType &s);
    
    MsgType GetType() const { return Type; }
    void SetParam(enum ParamType Typ, const DatatableType &data);
    
    const DatatableType MakeString() const;
    void UnmakeString(const DatatableType &s);
    
    void Debug() const;
    
    char          GetCharParam(ParamType key) const;
    unsigned      GetIntParam(ParamType key) const;
    DatatableType GetDataParam(ParamType key) const;

private:
    MsgType Type;
    typedef map<unsigned, DatatableType> paramap;
    
    paramap params;
};
